/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Hyperion Pattern.
 Hyperion Pattern is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/

#ifndef _hyp_ker_def_h
#define _hyp_ker_def_h


/*! \defgroup PlatformDef Platforms Definitions
\ingroup HypDef
Here are some definitions related to the developped platforms.
*/
//@{
#ifdef WIN32
	/*!
	Is defined if the project, which includes the <i>Hyperion Pattern</i>, is constructed on Win32 platforms.
	Only Win32 platforms are supported now.
	*/
	#define hyp_WIN32

	#ifdef _DEBUG
		#define hyp_DEBUG
	#else
		/*!
		Is defined if the project, which includes the <i>Hyperion Pattern</i>, is released.
		Else \c hyp_DEBUG is defined.
		*/
		#define hyp_RELEASE
	#endif
#else
	#error Only Win32 Plateform is supported. Portage on Linux should be easy ;-)
#endif
//@}

/*! \defgroup CompiloDef Compiler Definitions
\ingroup HypDef
Here are some definitions related to different compilers.
*/
//@{
#ifdef _MSC_VER
	/*!
	Is defined if the compilateur is MS Visual C++.
	*/
	#define hyp_VC _MSC_VER    
#else
	#error Only Visual C++ 6.0 is available.
#endif
//@}

/*! \defgroup CallingDef Calling Conventions
\ingroup HypDef
Definitions related to the calling convention used by default for exported functions.
If \c hyp_NO_CALL_CONV is defined, no special calling convention is used.

Report to your compiler for more information.
*/
//@{
#ifdef hyp_NO_CALL_CONV
	#define hyp_CALL_CONV
	#pragma message( "hyp_NO_CALL_CONV activated" )
#else
	#define hyp_CALL_CONV __stdcall
#endif
//@}

/*! \defgroup NameDef Namespaces
\ingroup HypDef
Here are some definitions related to the declarations of namespaces.
*/
//@{
//! Fast declaration of namespaces.
#define hyp_NAMESPACE_BEGIN( n )                    \
	namespace n {
//! See \ref hyp_NAMESPACE_BEGIN
#define hyp_NAMESPACE_END( n )                      \
	}

//! Used to insert some components in the Hyperion namespace.
#define hyp_NAMESPACE_BEGIN_HYP                     \
	hyp_NAMESPACE_BEGIN(hyp)
//! See \ref hyp_NAMESPACE_BEGIN_HYP
#define hyp_NAMESPACE_END_HYP                       \
	hyp_NAMESPACE_END(hyp)
//@}


/*! \defgroup DllDef Exportation Definitions
\ingroup HypDef
Here are some definitions used to export compounds in a dynamic linking library (i.e DLL).
*/
//@{
#ifdef hyp_WIN32
	//!	Is defined to export components from dlls.
	#define hyp_EXPORT_DLL __declspec( dllexport )
	//! Is defined to import components from dlls.
	#define hyp_IMPORT_DLL __declspec( dllimport )
#endif
//@}

//-------------------------------------------------------
// Inclusion of the standard libraries
//-------------------------------------------------------

#include<vector>
#include<list>
#include<iomanip>
#include<string>
#include<sstream>
#include<fstream>
#include<map>
#include<cassert>
#include<set>
#include<memory>


#ifdef hyp_USE_VALARRAY
#include<valarray>
#endif


#ifndef hyp_NO_INCLUDE_SYS_HEADER

//-------------------------------------------------------
// Inclusion of the system libraries
//-------------------------------------------------------
	#ifdef hyp_WIN32
		#define WIN32_LEAN_AND_MEAN // exclude rarely-used stuff from Windows headers
		#include<windows.h>
		#include<winnt.h>
		#include<windef.h>
		#include<winbase.h>
	#endif

#endif


#endif