/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Hyperion Pattern.
 Hyperion Pattern is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$

 Acknowledgements stx
 Creation 19/03/01 PR
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/

#include"hyp.kernel.core.h"

hyp_NAMESPACE_BEGIN_HYPKERNEL



/*!
\ingroup Assert
General function which allows to display a message box. Used for trace or debug operations.
\param Message String which contains the message to display.
\param MessageType Indicates which types of message to display.
*/
hyp_ker_DLL void Message(const std::string& Message, EMessageType MessageType)
{
	std::string title;
	switch(MessageType) {
	case notify: {
		title="NOTIFY ERROR";
		break;
	}
	case assertion: {
		title="ASSERTION FAILED";
		break;
	}
	case fatal: {
		title="FATAL ERROR";
		break;
	}
	default:
		hyp_ASSERT(0);
	}

	if(MessageType==fatal || MessageType==assertion) {
#ifdef hyp_WIN32
	::MessageBeep( MB_ICONEXCLAMATION );
	::MessageBox( NULL, Message.c_str(), title.c_str(), MB_ICONSTOP | MB_OK | MB_TASKMODAL | MB_SETFOREGROUND );
	::DebugBreak();
	::ExitProcess(1);
#endif
	} else {
#ifdef hyp_WIN32
		if(::MessageBox( NULL,
						(Message+std::string("\nDo you want to continue the application?")).c_str(),
						title.c_str(),
						MB_ICONWARNING | MB_YESNO | MB_TASKMODAL | MB_SETFOREGROUND ) == IDNO) {
			::DebugBreak();
			::ExitProcess(1);
		}
#endif
	}

}



hyp_NAMESPACE_END_HYPKERNEL