/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Hyperion Pattern.
 Hyperion Pattern is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$

 Acknowledgements stx
 Creation 19/03/01 PR
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/


#ifndef _hyp_ker_assert_h_
#define _hyp_ker_assert_h_

#ifndef hyp_KERNEL_CORE_INSIDE
#error Inclusion not allowed (include hyp.kernel.core.h instead)
#endif

/*!\ingroup Assert
This enumeration is only used with the Message function.
*/
enum EMessageType {
	notify,		//!< Indicates that the user will have the posssibility to continue the application.
	fatal,		//!< Indicates that the application are going to exit.
	assertion	//!< Indicates that Message function is used to display assertion information.
};

hyp_ker_DLL void Message(const std::string&,EMessageType);

#ifdef hyp_DEBUG
/*!
\ingroup Assert
Custon assertion macro.
*/
#define hyp_ASSERT( Condition )                                 \
{                                                               \
	if ( !( Condition ) ) {                                     \
		std::string formated_msg;                               \
		hyp_STRING_TRACE( formated_msg , ("%s",#Condition) );   \
		hyp::ker::Message( formated_msg , hyp::ker::assertion);        \
	}                                                           \
}

#else
#define hyp_ASSERT( Condition )
#endif

/*!
\ingroup Assert
Displays a message box and kills the applications. Uses when an error
can not be recovered.
*/
#define hyp_FATAL( Msg )                                         \
{                                                                \
	std::string formated_msg;                                    \
	hyp_STRING_TRACE( formated_msg , Msg );                      \
	hyp::ker::Message( formated_msg, hyp::ker::fatal);                  \
}

/*!
\ingroup Assert
Display a message box which proposes to quit or to continue the application.
*/
#define hyp_NOTIFY( Msg )                                        \
{                                                                \
	std::string formated_msg;                                    \
	hyp_STRING_TRACE( formated_msg , Msg );                      \
	hyp::ker::Message( formated_msg, hyp::ker::notify );                \
}


#endif