/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Hyperion Pattern.
 Hyperion Pattern is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$

 Creation 10/05/01 PR
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/


#ifndef _hyp_ker_command_h_
#define _hyp_ker_command_h_

#ifndef hyp_KERNEL_CORE_INSIDE
#error Inclusion not allowed (include hyp.kernel.core.h instead)
#endif


class hyp_ker_DLL CCmdInfoArg
{
protected:
	static int m_Argc;
	static char** m_Argv;
	/*!
	Initializes the static data for the command line. This initialization is done 
	with the CCmdInfo::CCmdInfo constructor.
	*/
	inline static void SetArg(int Argc,char** Argv) { m_Argc=Argc; m_Argv=Argv; }
};


template<class T>
class CCmdInfoData :
protected CCmdInfoArg
{
public:
	virtual int GetSize() const =0;
	virtual T GetValue(int) const =0;
};

class hyp_ker_DLL CCmdInfoLabel :
public CCmdInfoData<std::string>
{
	int m_PosBegin;		//postion du label
	int m_PosLast;	//position du label suivant
public:
	CCmdInfoLabel(int =0,int =0);
	virtual int GetSize() const;
	virtual std::string GetValue(int) const;

	/*!
	Gets the value of a parameter.
	\param Value Value of the parameter.
	\param Pos Position of the parameter.
	*/
	template<class T>
	void Get(T& Value,int Pos) const {
		std::stringstream str_stream;
		str_stream<<GetValue(Pos);
		str_stream>>Value;
	}
};


class CCmdInfo;

typedef std::vector<CCmdInfoLabel> t_CmdInfoLabelEnum;


class hyp_ker_DLL CCmdInfoLabelEnum :
public CCmdInfoData<CCmdInfoLabel>,
public t_CmdInfoLabelEnum
{
	std::string m_LabelName;
public:
	CCmdInfoLabelEnum(const std::string&);
	virtual int GetSize() const;
	virtual CCmdInfoLabel GetValue(int) const;
	/*!
	\return Label name of the enumeration.
	*/
	inline std::string GetLabelName() const { return m_LabelName; }	
};


class hyp_ker_DLL CCmdInfo :
public CCmdInfoData<CCmdInfoLabelEnum>
{
	const char m_LabelKey;
	
public:
	CCmdInfo(int,char**,char ='-');
	virtual int GetSize(void) const;
	virtual CCmdInfoLabelEnum GetValue(int) const;	
	std::string GetModuleFileName() const;
	std::string GetModulePath() const;
	std::string GetModuleName() const;
	std::string GetLineOfParameters() const;

private:
	int GetNextLabelPosition(int) const;
	std::string GetLabelName(int) const;
	int GetNextLabel(const std::string&,int) const;

	friend hyp_ker_DLL std::ostream& operator<<(std::ostream& ,const CCmdInfo&);
};


#endif