/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Hyperion Pattern.
 Hyperion Pattern is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/



#ifndef _hyp_ker_dll_h_
#define _hyp_ker_dll_h_

#ifndef hyp_KERNEL_CORE_INSIDE
#error Inclusion not allowed (include hyp.kernel.core.h instead)
#endif


/*!
\ingroup Dll
Indicates why the InitLibrary function has been called.
*/
enum EDllReason {
	dll_attach,	//!< The library has been loaded.
	dll_detach	//!< The library are goind to be unloaded.
};

/*!
\ingroup Dll
Pointer to the InitLibrary function. This function is the equivalent to the WinDll for the
Win32 platforms.
\code
void CALL_CONV InitLibrary(const std::vector<std::string>& Param,EDllReason Reason);
\endcode
\param Param A string array contained data. Hyperion Pattern imposes that the first element
of the array containts the name of the loaded library.
\param Reason The reason explained why the function has been called.
*/
typedef void (hyp_CALL_CONV *PTR_INITLIBRARY)(const std::vector<std::string>&,EDllReason);



class hyp_ker_DLL CDllUtility
{
	class hyp_ker_DLL CDllData
	{
		bool m_InitLibrary;
		HINSTANCE m_H;
	public:
		CDllData(HINSTANCE,bool);
		CDllData(const CDllData&);
		CDllData();
		HINSTANCE& GetH();
		const HINSTANCE& GetH() const;
		bool Init() const;
		bool FreeLibrary();
	};
	
	typedef std::map<std::string,CDllData> t_DllCont; 
	struct hyp_ker_DLL CDllContainer :
	public t_DllCont
	{
		virtual ~CDllContainer();
	};
	
	static CDllContainer ms_DllCont;

	void InitializeLibrary(const std::string&,EDllReason);	

public:
	void LoadLibrary(const std::string&,bool);
	void UnloadLibrary(const std::string&);
	void UnloadAllLibraries();
	void* GetFunctionAddress(const std::string&,const std::string&);
	
	hyp_ker_DLL friend std::ostream& operator<<(std::ostream&,const CDllUtility&);
};


#endif
