/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Hyperion Pattern.
 Hyperion Pattern is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/



#include"hyp.kernel.core.h"

hyp_NAMESPACE_BEGIN_HYPKERNEL



//-----------------------------------------------------------------------
// CComResultDescriptor
//-----------------------------------------------------------------------

class CComResultDescriptor
{
	typedef std::map<com_result,std::string> t_ContResultDescription;
	t_ContResultDescription m_MapResultDescription;
	
	void DefineResult(com_result Result,const char* Descrip)	{
		m_MapResultDescription[Result]=Descrip;
	}

public:
	CComResultDescriptor() {
	DefineResult(com_ok,"True - no error");
	DefineResult(com_notified,"Result notified - no error");
	DefineResult(com_false,"False");
	DefineResult(com_assertion_failed,"Assertion failed");
	DefineResult(com_unexpected,"Catastrophic failure");
	DefineResult(com_not_impl,"Not implemented");
	DefineResult(com_no_interface,"No such interface supported");
	DefineResult(com_bad_arguments,"Bad choice with the arguments");
	DefineResult(com_no_available,"No implementation defined");
	DefineResult(com_must_no_call,"The function must not be called, it is not make any sense");
	DefineResult(com_no_aggregation,"Class does not support aggregation");
	DefineResult(com_class_not_available,"Class factory cannot supply requested class");
	DefineResult(com_out_of_memory,"Ran out of memory");
	DefineResult(com_invalid_arg,"One or more arguments are invalid");
	DefineResult(com_logic_in_arguments,"No logic with arguments");
	DefineResult(com_initialisation_data,"Error with the data initialization");
	DefineResult(com_bad_internal_data,"Bad internal data detected");
	DefineResult(com_directxfile,"Error occured with a DirectX component - Windows only");
	DefineResult(com_ios,"Input/output error");
	DefineResult(com_envvar,"Error about environment variables");
	DefineResult(com_file_missing,"File missing");
	DefineResult(com_winregistry,"Error about the registry - Windows only");
	}

	std::string GetResultDescription(com_result Result) {
		t_ContResultDescription::const_iterator itor=m_MapResultDescription.find(Result);
		return itor==m_MapResultDescription.end() ? "Result not documentated" : (*itor).second;
	}
};


CComResultDescriptor gs_ResultDescriptor;

//-----------------------------------------------------------------------
/*!
\class CException
\ingroup Exception
Root Class for the Hyperion exceptions.
Could be instanciated to rise exceptions.Each exception is documented by a string.
*/
//-----------------------------------------------------------------------

const char* CException::ms_DefaultMessage="Exception not documented";

CException::CException(const char* Doc) :
m_Doc(Doc)
{

}

CException::CException(const CException& Excep) :
m_Doc(Excep.m_Doc)
{

}

/*!
Give the full documentation of the object. 
*/
std::string CException::What(void) const
{
	return m_Doc;
}

/*!
Give the type of exception. Redefined by each exception class.
*/
std::string CException::Type(void) const
{
	return "Basic Exception";
}

hyp_ker_DLL std::ostream& operator<<(std::ostream& Stream,const CException& Excep)
{
	Stream<<"Exception type : "<<Excep.Type()<<"\n"
		  <<"Documentation : "<<Excep.What();
	return Stream;
}


//-----------------------------------------------------------------------
/*!
\class CSysException
\ingroup Exception
Generally generated when a system error occurs. It is actually when an API call fails.
*/
//-----------------------------------------------------------------------

/*!
During its creation, \c GetLastSysError is called to bring some extra information.
This information is added to the initial documentation of the exception.
*/
CSysException::CSysException(const char* Doc) :
CException(Doc)
{
	m_Doc+="Previous System Error : ";
	m_Doc+=GetLastSysError();
	m_Doc+="\n";
}

std::string CSysException::Type(void) const
{
	return "System Exception";
}

//-----------------------------------------------------------------------
/*!
\class CComException
\ingroup ComException
Class primary destinated for the Hyperion Component.
Try to mix the mechanisms of the C++ exception and the result code of the COM model.
So each exception class is identified by a result code.
*/
//-----------------------------------------------------------------------

/*!
Get the description of the result code.
\return Description of the result code.
*/
std::string CComException::GetResultDescription(void) const
{
	return gs_ResultDescriptor.GetResultDescription(m_Result);
}

/*!
If the result code designed a failed code, the full documentation of the exception
will be traced in the CERR file.
The full documentation is composed by :
- the documentation of the code result
- the complementary documentation corresponded to m_Doc member
\param Result the result code of the object
\param pDoc the complementary documentation of the exception
*/
CComException::CComException(com_result Result,const char* Doc) :
CException(Doc),
m_Result(Result)
{
	//FIXTODO : est ce joli?
	if(FAILED(Result))	cerr<<What();
}

std::string CComException::Type(void) const
{
	return "Component Exception";
}

/*!
The exception generated by the copy constructor doesn't write its error description
in the CERR file.
*/
CComException::CComException(const CComException& Error) :
CException(Error.m_Doc.c_str()),
m_Result(Error.m_Result)
{
	
}

std::string CComException::What(void) const
{
	std::string descrip("com_result=");
	descrip+=GetResultDescription();
	descrip+=m_Doc;
	descrip+='\n';
	
	return descrip;
}


hyp_NAMESPACE_END_HYPKERNEL