/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Hyperion Pattern.
 Hyperion Pattern is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/

#if !defined(_hyp_exception_h_)
#define _hyp_exception_h_

#ifndef hyp_KERNEL_CORE_INSIDE
#error Inclusion not allowed (include hyp.kernel.core.h instead)
#endif


class hyp_ker_DLL CException
{
protected:
	std::string m_Doc;
	static const char* ms_DefaultMessage;

public:
	CException(const char* =ms_DefaultMessage);
	CException(const CException&);
	
	virtual std::string What(void) const;
	virtual std::string Type(void) const;
};

hyp_ker_DLL std::ostream& operator<<(std::ostream&,const CException&);

class hyp_ker_DLL CSysException :
public CException
{
public:
	CSysException(const char* =ms_DefaultMessage);
	virtual std::string Type(void) const;
};


#define hyp_THROW_ASSERT( Exception , Condition ) \
	if(!Condition) hyp_THROW2( Exception , (#Condition) );


/*!
\ingroup Exception
*/
#define hyp_THROW( Exception )								\
{															\
	std::string doc;										\
	hyp_STRING_BEGIN( doc );								\
	hyp_STRING_END( doc );									\
	throw Exception(doc.c_str());									\
}


/*!
\ingroup Exception
*/
#define hyp_THROW2( Exception , Args )						\
{															\
	std::string doc;										\
	hyp_STRING_BEGIN( doc );								\
	hyp_STRING_DO( doc, Args );								\
	hyp_STRING_END( doc );									\
	throw Exception(doc.c_str());									\
}


class hyp_ker_DLL CComException :
public CException
{
	com_result m_Result;
public:
	CComException(com_result= com_false,const char* =ms_DefaultMessage);
	CComException(const CComException&);

	virtual std::string What(void) const;
	virtual std::string Type(void) const;

	inline bool operator==(const com_result& Result) const {
		return (m_Result==Result);
	}
	inline bool operator!=(const com_result& Result) const {
		return !(*this==m_Result);
	}
	inline operator com_result() const {
		return GetResult();
	}

	//! Get the result code
	inline com_result GetResult(void) const {
		return m_Result;
	}
	inline bool Failed() const {
		return ker::Failed(m_Result);
	}
	inline bool Succeeded() const {
		return ker::Succeeded(m_Result);
	}
	std::string GetResultDescription(void) const;
};


#define hyp_THROW_COM_ASSERT( Condition ) \
	if(!Condition) hyp_THROW_COM2( hyp_ker::com_assertion_failed, (#Condition) );


/*!
\ingroup Exception
*/
#define hyp_THROW_COM2( Result , Args )						\
{															\
	std::string doc;										\
	hyp_STRING_BEGIN( doc );								\
	hyp_STRING_DO( doc , Args );							\
	hyp_STRING_END( doc );									\
	throw hyp::ker::CComException(Result,doc.c_str());	\
}

/*!
\ingroup Exception
*/
#define hyp_THROW_COM( Result )								\
{															\
	std::string doc;										\
	hyp_STRING_BEGIN( doc );								\
	hyp_STRING_END( doc );									\
	throw hyp::ker::CComException(Result,doc.c_str());		\
}

#define hyp_CATCH_COM_ITOR                           \
	catch(hyp::ker::CComException& Except) {             \
		if( Except.Failed() ) throw Except;          \
	}

#endif