/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Hyperion Pattern.
 Hyperion Pattern is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/

#include"hyp.kernel.core.h"

hyp_NAMESPACE_BEGIN_HYPKERNEL


#pragma data_seg(".Shared")
std::fstream* p_cerr=0;
/*!
\ingroup GlobalComp
Defines the cerr file.
*/
hyp_ker_DLL std::fstream& cerr=*p_cerr;

std::fstream* p_clog=0;
/*!
\ingroup GlobalComp
Defines the clog file.
*/
hyp_ker_DLL std::fstream& clog=*p_clog;

#pragma data_seg()

//----------------------------------------------------------
/*! \class CGlobal
\ingroup GlobalComp
Used to initialize the global variables of the framework.
*/
//----------------------------------------------------------


int CGlobal::m_Count=0;

/*!
Inits the cerr and the log file.
*/
CGlobal::CGlobal()
{
	if(m_Count++==0) {
		try{
			const char* path=::getenv("HYP_CERR");
			if(!path) hyp_THROW2( CException , ("Environment variable HYP_CERR doesn't exist") );
		
			std::string name_file;
			CDir dir_global_stream;
			dir_global_stream.Open(path);

			name_file=std::string(path)+"hyp_cerr";
			dir_global_stream.CreateFile("hyp_cerr");
			p_cerr=new std::fstream(name_file.c_str());

			name_file=std::string(path)+"hyp_clog";
			dir_global_stream.CreateFile("hyp_clog");
			p_clog=new std::fstream(name_file.c_str());		
		
		} catch(CException excep) {
			hyp_FATAL( ("LOG and CERR file can not be created. %s",excep.What().c_str()) );
		}
	}
}

/*!
Closes the cerr and the clog files if these files are not used.
\todo comment fixer  la compilation l'ordre de destruction
*/
CGlobal::~CGlobal()
{
	if(--m_Count==0) {

		//nettoyage du flux
		if(&cerr) {
			cerr.close();
			delete &cerr;
		}
		if(&clog) {
			clog.close();
			delete &clog;
		}
	}
}

/*!
\ingroup GlobalComp
Returns the last formated error descriptions. This error is a system error.
*/
hyp_ker_DLL std::string GetLastSysError(void)
#ifdef hyp_WIN32
{
	void* pMsgBuf;
	
	::FormatMessage(
		FORMAT_MESSAGE_ALLOCATE_BUFFER|FORMAT_MESSAGE_FROM_SYSTEM,
		NULL,
		::GetLastError(),
		MAKELANGID(LANG_NEUTRAL,SUBLANG_DEFAULT),
		(LPTSTR)&pMsgBuf,
		0,
		NULL);

	std::string str_result;

	if(pMsgBuf!=NULL) {
		str_result+=static_cast<char*>(pMsgBuf);
		LocalFree(pMsgBuf); 
	}

	return str_result;		
}
#endif

/*!
\ingroup GlobalComp
CVS give the version of the Hyperion Kernel Library.
*/
hyp_ker_DLL std::string GetLibraryVersion(void)
{
	const char* library_version="Hyperion Library (c) 2000-2001 Pierre Rebours\n\
$Id$\n\
See http://lagrandeporte.multimania.com/hyperion";
	
	return library_version;
}

hyp_NAMESPACE_END_HYPKERNEL