/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Hyperion Pattern.
 Hyperion Pattern is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/


#include"hyp.kernel.core.h"

hyp_NAMESPACE_BEGIN_HYPKERNEL


//---------------------------------------------------------
/*! \class CGuid
\ingroup Guid
The aim of CGuid is to encapsulate the notion of the General Universal ID (i.e. GUID).
In the COM environment, the GUID is a 128-bit value that uniquely identifies objects such 
as OLE servers, interfaces, manager entry-point vectors, and client objects.

Win32 plateform presented the GUID with the following structure (cf.\c winnt.h) :
\code
	typedef struct _GUID {
			DWORD Data1;
			WORD   Data2;
			WORD   Data3;
			BYTE  Data4[8];
		} GUID;	
\endcode
with:
- DWORD, a 32-bit unsigned integer or the address of a segment and its associated offset
- WORD, a 16-bit unsigned integer
- BYTE, an 8-bit integer that is not signed.
*/
//---------------------------------------------------------

/*!
Constructor. Do not use directly this function, but use the hyp_DEFINE_GUID macro.
*/
CGuid::CGuid(t_ulong data1,
		t_ushort data2,t_ushort data3,
		t_uchar data4_0,t_uchar data4_1,t_uchar data4_2,t_uchar data4_3,
		t_uchar data4_4,t_uchar data4_5,t_uchar data4_6,t_uchar data4_7)
{
	m_Guid.Data1=data1;
	m_Guid.Data2=data2;m_Guid.Data3=data3;
	m_Guid.Data4[0]=data4_0;m_Guid.Data4[1]=data4_1;
	m_Guid.Data4[2]=data4_2;m_Guid.Data4[3]=data4_3;
	m_Guid.Data4[4]=data4_4;m_Guid.Data4[5]=data4_5;
	m_Guid.Data4[6]=data4_6;m_Guid.Data4[7]=data4_7;
}

/*!
Initializes all the data to 0.
\warning Use this constructor with carefulness and only temporarily.
*/
CGuid::CGuid()
{
	memset(&m_Guid,0,sizeof(m_Guid));
}

//FIXTODO enlever la variable ul_buff
/*!
Initializes the object with the string. This string must be formated like Guiden.exe.
See CGuid::ToString.
*/
CGuid::CGuid(const std::string& String)
{
	memset(&m_Guid,0,sizeof(m_Guid));
	
	std::stringstream buffer;
	t_ulong ul_buff;

	buffer<<"0x"<<std::string(String,1,8)<<'\n';
	buffer>>std::hex>>m_Guid.Data1;

	buffer<<"0x"<<std::string(String,10,4)<<'\n';
	buffer>>std::hex>>m_Guid.Data2;

	buffer<<"0x"<<std::string(String,15,4)<<'\n';
	buffer>>std::hex>>m_Guid.Data3;

	buffer<<"0x"<<std::string(String,20,2)<<'\n';
	buffer>>std::hex>>ul_buff;
	m_Guid.Data4[0]=ul_buff;

	buffer<<"0x"<<std::string(String,22,2)<<'\n';
	buffer>>std::hex>>ul_buff;
	m_Guid.Data4[1]=ul_buff;;

	buffer<<"0x"<<std::string(String,25,2)<<'\n';
	buffer>>std::hex>>ul_buff;
	m_Guid.Data4[2]=ul_buff;

	buffer<<"0x"<<std::string(String,27,2)<<'\n';
	buffer>>std::hex>>ul_buff;
	m_Guid.Data4[3]=ul_buff;

	buffer<<"0x"<<std::string(String,29,2)<<'\n';
	buffer>>std::hex>>ul_buff;
	m_Guid.Data4[4]=ul_buff;

	buffer<<"0x"<<std::string(String,31,2)<<'\n';
	buffer>>std::hex>>ul_buff;
	m_Guid.Data4[5]=ul_buff;

	buffer<<"0x"<<std::string(String,33,2)<<'\n';
	buffer>>std::hex>>ul_buff;
	m_Guid.Data4[6]=ul_buff;

	buffer<<"0x"<<std::string(String,35,2)<<'\n';
	buffer>>std::hex>>ul_buff;
	m_Guid.Data4[7]=ul_buff;	
}

/*!
Converts the object to a string.
This string is generated with the same format like Guiden.exe.
\code
{FCA9BB80-69A5-11d5-A099-0080AD97951B}
\endcode
\return The formated string.
*/
std::string CGuid::ToString(void) const
{
	std::stringstream stream;
	stream<<'{'<<std::hex<<std::setfill('0')<<std::uppercase
		<<std::setw(8)<<m_Guid.Data1<<'-'
		<<std::setw(4)<<m_Guid.Data2<<'-'
		<<std::setw(4)<<m_Guid.Data3<<'-'
		<<std::setw(2)<<static_cast<int>(m_Guid.Data4[0])
		<<std::setw(2)<<static_cast<int>(m_Guid.Data4[1])<<'-'
		<<std::setw(2)<<static_cast<int>(m_Guid.Data4[2])
		<<std::setw(2)<<static_cast<int>(m_Guid.Data4[3])
		<<std::setw(2)<<static_cast<int>(m_Guid.Data4[4])
		<<std::setw(2)<<static_cast<int>(m_Guid.Data4[5])
		<<std::setw(2)<<static_cast<int>(m_Guid.Data4[6])
		<<std::setw(2)<<static_cast<int>(m_Guid.Data4[7])
		<<'}';
		
	return stream.str();
}


hyp_NAMESPACE_END_HYPKERNEL
