/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Hyperion Pattern.
 Hyperion Pattern is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/


#ifndef _hyp_objbase_h_
#define _hyp_objbase_h_

#ifndef hyp_KERNEL_INSIDE
#error Inclusion not allowed (include hyp.kernel.h instead)
#endif


/*!
\ingroup ExportedFunctions
Gives a pointer to a component factory.
\code
com_result hyp_CALL_CONV hypDllGetClassObject(const hyp_ker::hypCLSID& clsid,const hyp_ker::hypIID& iid,void** ppv);
\endcode
\param CLSID CLSID of the component that will be created by the factory.
\param IID IID of the factory interface.
\param ppv Pointer to the factory interface.
*/
typedef com_result (hyp_CALL_CONV *PTR_HYPDLLGETCLASSOBJECT)(const CGuid& CLSID,const CGuid& IID,void** ppv);

/*!
\ingroup ExportedFunctions
Checks if the component server can be unloaded.
\code
com_result hyp_CALL_CONV hypDllCanUnloadNow(void);
\endcode
\return If the result code is succeeded, the component server can be unloaded.
*/
typedef com_result (hyp_CALL_CONV *PTR_HYPDLLCANUNLOADNOW)(void);

/*!
\ingroup ExportedFunctions
Registers all the componant contained in the server.
\code
com_result hyp_CALL_CONV hypDllRegisterServer(void);
\endcode
\return Result code.
*/
typedef com_result (hyp_CALL_CONV *PTR_REGISTERSERVER)(void);

/*!
\ingroup ExportedFunctions
Unregisters all the componant contained in the server.
\code
com_result hyp_CALL_CONV hypDllUnregisterServer(void);
\endcode
\return Result code.
*/
typedef com_result (hyp_CALL_CONV *PTR_UNREGISTERSERVER)(void);


/*!
\ingroup Component
Designed to stock informations about the server.
Used only for compatibility with COM.
*/
typedef int hypSERVERINFO;

hyp_ker_DLL extern const t_ulong INPROC_SERVER;

hyp_ker_DLL com_result hypGetPathInprocessServer(const CGuid&, std::string&);

hyp_ker_DLL com_result CreateInstance(const hypCLSID&,IUnknown*,
				  t_ulong,const hypIID&,void**);
hyp_ker_DLL com_result GetClassObject(const hypCLSID&, t_ulong, hypSERVERINFO*,
				  const hypIID&,void**);
hyp_ker_DLL void FreeUnusedLibraries(void);

hyp_ker_DLL void GetLoadedLibraries(std::ostream&);

#endif