/*
***************************************************************************************************
***************************************************************************************************
***
***    File Name: WFProfiler.cpp
***  Description: Functional implementation of the WFProfiler.h macros and function declarations. 
***        Notes: These macros are specific to the Intel Pentium processors. Also, the code
***               is specific to the MS Visual C++ compiler due to the __int64 declarations.
*** 
***               Unfortunately, these macros are useless to determine small operations such as
***               single shifts, single adds etc. For some reason, they reflect as taking -2 
***               clock cycles to complete. Is this due to caching? Multi-processing? Who knows.
***               When testing from Start()-Stop() with no instructions in between, I repeatedly
***               recieve a 0-cycle count as expected. Intel even states that the RDTSC instruction
***               is not perfect - especially when combined with CPUID. Check out the web-site:
***
***                     http://developer.intel.com/ for more information. (Search for RDTSC)...
***
***       Author: J.W. Heikkila, Jr.
***
***    Copyright:
***
***                 Confidential and Proprietary Source Code for
***                      Wild-Fire Gaming Systems, Inc.
***                     437 Jamesway, Marion, Ohio 43302
***
***    Revisions:
***
***              10/05/2000 - JWH - Original version.
***
***    Copyright (c) 2000, Wild-Fire Gaming Systems, Inc. All rights reserved.
***
***************************************************************************************************
***************************************************************************************************
*/

//hyp.ker.profiler.cpp
//inclusion de WfProfiler dans le kernel d'Hyperion 02/05/01 Tycho
//modification 02/05/01 Tycho

#include"hyp.ker.extern.h"

hyp_NAMESPACE_BEGIN_HYPKERNEL

hyp_ker_DLL void Clear(WF_Profiler& data)
{
	memset(&data,0,sizeof(data));
}


// Resets all values...
hyp_ker_DLL void Reset(WF_Profiler& data)
{
  // We need to clear if nothing was initialized...
  if (1!=data.Started) Clear(data);
}


// How many cycles was it...
hyp_ker_DLL t_int64 Duration(const WF_Profiler& data)
{                                                                     
  if (1 == data.Started) return 0;                        // Error! Still working...
  t_int64 baseEnd = ((t_int64) data.EndHigh << 32) | data.EndLow;
  t_int64 baseStart = ((t_int64) data.StartHigh << 32) | data.StartLow;
  return (baseEnd - baseStart) - data.Base;                // We're done. Return the total...
}



// Calculates the length of time it takse from the RDTSC in StartNow() until the RDTSC
// in StopNow(). Only good if run simultaneously (see Start() below)...
hyp_ker_DLL void SetDuration(WF_Profiler& data)       
{               
  if (0 == data.Started)
  {
    t_int64 baseEnd = ((t_int64) data.EndHigh << 32) | data.EndLow;
    t_int64 baseStart = ((t_int64) data.StartHigh << 32) | data.StartLow;
    unsigned newBase = (unsigned) (baseEnd - baseStart);
    if (0==data.Base) data.Base = newBase;
     else  data.Base = (data.Base <= newBase) ? data.Base : newBase;   // Keep the lowest value...
  }                                              
}

hyp_NAMESPACE_END_HYPKERNEL