/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Hyperion Pattern.
 Hyperion Pattern is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/

#include"hyp.kernel.core.h"

hyp_NAMESPACE_BEGIN_HYPKERNEL



//----------------------------------------------------------
/*! \class CStdRegistry
\ingroup StdRegistry
Emulate the Registry System of the Win32 platforms. It uses the files
throught the standard library.
*/
//----------------------------------------------------------

/*!
Init the Registry
\param Name Path of the registry root. If the path doesn't exist, it is created.
*/
CStdRegistry::CStdRegistry(const std::string& Name)
{
	m_pCurrentRegistryFile=0;
	
	//on rcupre le chemin du registry et on concasse avec Path
	const char* path=::getenv("HYP_REG");
	if(!path) hyp_FATAL( ("Environment variable HYP_REG doesn't exist") );
	std::string path_temp(path);
	path_temp+=Name;
	Open(path_temp,true);	
}


CStdRegistry::~CStdRegistry()
{
	if(m_pCurrentRegistryFile)
		delete m_pCurrentRegistryFile;
}



/*!
Returns the current pointer to the current Registry File.
\param Name Name of the Registry File that will be the current Registry File of the object.
\return Pointer to the current Registry File. If an error occurs, an exception is thrown, so
the return pointer is always valid.
*/
CStdRegistryFile* CStdRegistry::GetRegistryFile(const std::string& Name)
{
	if(!m_pCurrentRegistryFile) delete m_pCurrentRegistryFile;
	m_pCurrentRegistryFile=0;
	
	return m_pCurrentRegistryFile=new CStdRegistryFile(Name,this);
}

/*!
Create a file and a Registry File object wrapping this file.
The object become the default Registry File of the object.
\param Name Name of the file which are going to be created.
\return  A new Registry File object which pointers to the created file.
\note The missing paths are been created.
*/
CStdRegistryFile* CStdRegistry::CreateRegistryFile(const std::string& Name)
{
	CreateFile(Name);
	return GetRegistryFile(Name);
}

/*!
Deletes the file symbolised by the object CStdRegistryFile(Path,Name,this)
\param Name Name of the file which are going to be deleted
*/
void CStdRegistry::DeleteRegistryFile(const std::string& Name)
{
	DeleteFile(Name);
}



hyp_NAMESPACE_END_HYPKERNEL