/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Hyperion Pattern.
 Hyperion Pattern is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/

#include"hyp.kernel.core.h"

hyp_NAMESPACE_BEGIN_HYPKERNEL


const std::string g_SEPARATOR("->");

//---------------------------------------------------------
/*! \class CStdRegistryFile
\ingroup StdRegistry
Emulates the notion of the Win32 Registry Keys
*/
//---------------------------------------------------------

CStdRegistryFile::CStdRegistryFile(const std::string& Name,CStdRegistry *pRegistry) :
m_Name(Name),
m_pRegistry(pRegistry)
{
	//on vrifie que le fichier existe
	if (!pRegistry->IsInRegistry(Name)) hyp_THROW2( CException , ("%s doesn't exist",Name.c_str()) );
	
	std::ifstream file((pRegistry->GetPathofRegistry()+Name).c_str());
	if(!file) hyp_THROW( CException );
	char ch;
	while(file.get(ch)) m_FileString+=ch;
	if(!file.eof()) hyp_THROW( CException );	
}

CStdRegistryFile::~CStdRegistryFile()
{
	std::ofstream file((m_pRegistry->GetPathofRegistry()+m_Name).c_str());
	if(file) {
		file<<m_FileString;
	}
}

/*!
Get the value in the key
*/	
std::string CStdRegistryFile::GetValue(const std::string& Key) const
{
	std::string::size_type pos_end=0,pos_first=0;
		
	if(!GetPosValue(Key,pos_first,pos_end)) hyp_THROW( CException );
	
	return std::string(m_FileString,pos_first,pos_end-pos_first);	
}

/*!
Associates a value to the key. The previous value is erased.
*/
void CStdRegistryFile::SetValue(const std::string& Key, const std::string & Value)
{
	std::string::size_type pos_end=0,pos_first=0;
		
	if(!GetPosValue(Key,pos_first,pos_end)) hyp_THROW( CException );
	DeletePartofStream(Value,pos_first,pos_end);	
}


/*!
Creates a new key.
- If the key is created, it is putted to the end
- If the key is already created, the file is not modified.
*/
void CStdRegistryFile::SetKey(const std::string& Key)
{
	std::string::size_type pos_end=0,pos_first=0;
		
	if(!GetPosKey(Key,pos_first,pos_end)) {
		//il y a pas de cl, on en cr  la fin
		//if(m_pStringStream->size()&&*((*m_pStringStream).end()-1)!='\n')
		//	*m_pStringStream+='\n';
		m_FileString+=Key;
		m_FileString+=g_SEPARATOR;
		m_FileString+='\n';
	}
}

/*!
Delete the key and its associted value.
*/
void CStdRegistryFile::DeleteKey(const std::string& Key)
{
	std::string::size_type pos=0;
	std::string::size_type pos_end_value=0,pos_first_key=0;
		
	if(GetPosKey(Key,pos_first_key,pos)) {
		if(GetPosValue(Key,pos,pos_end_value)) {
			DeletePartofStream("",pos_first_key,pos_end_value);			
		}
	}
}


bool CStdRegistryFile::GetPosValue(const std::string& Key,
					std::string::size_type& First,
					std::string::size_type& End) const
{
	std::string::size_type pos_end=0,pos_first=0;
	bool breturn=false;
	
	if(GetPosKey(Key,pos_first,pos_end)) {
		//on donne la position du prochain saut de ligne
		//assume qu'il tombe sur '\n' ou eof
		//FIXTODO qd on est  lafin, retourne valeur !=npos
		pos_first=m_FileString.find('\n',pos_end);
		pos_end+=g_SEPARATOR.length()==1 ? 1 : g_SEPARATOR.length();
		First=pos_end;
		End=pos_first;
		breturn=true;		
	}
	return breturn;
}

bool CStdRegistryFile::GetPosKey(const std::string& Key,
							   std::string::size_type& First,
							   std::string::size_type& End) const
{
	std::string::size_type pos_end=0,pos_first=0;
	bool breturn=false;
	
	while(1) {
		pos_end=m_FileString.find(g_SEPARATOR,pos_first);
		if(pos_end==std::string::npos) {
			//on a rien trouv
			break;
		}
		
		//on copie la cl
		std::string key_prov(m_FileString,pos_first,pos_end-pos_first);
		if(key_prov==Key) {
			breturn=true;
			First=pos_first;
			End=pos_end;
			break;
		} else {
			pos_first=m_FileString.find('\n',pos_end);
			//included '\n'
			pos_first++;
		}
	}

	return breturn;
}

void CStdRegistryFile::DeletePartofStream(const std::string& Value,
		const std::string::size_type& First,
		const std::string::size_type& End)
{
	//supprimer au milieu d'une string
	std::string buff(m_FileString,0,First);
	buff+=std::string(m_FileString,End,-1);
	buff.insert(First,Value);
	m_FileString=buff;
}

hyp_NAMESPACE_END_HYPKERNEL