/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Hyperion Pattern.
 Hyperion Pattern is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$

 Acknowledgements
 Some helper functions were borrowed and modifed from Kraig Brockschmidt's book Inside OLE.
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/


#include"hyp.kernel.core.h"

hyp_NAMESPACE_BEGIN_HYPKERNEL


#ifdef hyp_WIN32
//------------------------------------------------------------------------------
/*! \class CWin32ImplRegistry
\ingroup SysRegistry
Implements the IImplRegistry for the Win32 registry.
*/
//------------------------------------------------------------------------------

/*!
Set the root by default at \code //HKEY_LOCAL_MACHINE\\Software\\Hyperion \endcode
*/
CWin32ImplRegistry::CWin32ImplRegistry()
{
	// Create and open key and subkey.
	long lResult =::RegCreateKeyEx(HKEY_LOCAL_MACHINE,"Software\\Hyperion",
		0, NULL, REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS, NULL,
		&m_hRootKey, NULL);

	if(lResult!=ERROR_SUCCESS) hyp_THROW( CSysException );
}

CWin32ImplRegistry::~CWin32ImplRegistry()
{
	::RegCloseKey(m_hRootKey);
}


std::string CWin32ImplRegistry::GetAbsoluteKey(const std::string& ParentKey,
											   const std::string& ChildKey)
{
	std::string absolute_key(ParentKey);
	absolute_key+='\\';
	absolute_key+=ChildKey;

	return absolute_key;
}

bool CWin32ImplRegistry::GetHandleKey(const char* Key,const char* Subkey,HKEY* phSubkey)
{
	long lResult=::RegCreateKeyEx(m_hRootKey,
	                              GetAbsoluteKey(Key,Subkey).c_str(),
	                              0, NULL, REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS, NULL, 
	                              phSubkey, NULL);

	return lResult == ERROR_SUCCESS ? true : false;
}

/*!
Deletes a key and all of its descendents.
\param hKeyParent Parent of key to delete.
\param KeyChild Key to delete
*/
bool CWin32ImplRegistry::RecursiveDeleteKey(HKEY hKeyParent,const char* KeyChild)
{
	// Open the child.
	HKEY hKeyChild ;
	LONG lRes = RegOpenKeyEx(hKeyParent, KeyChild, 0,
	                         KEY_ALL_ACCESS, &hKeyChild) ;
	if (lRes != ERROR_SUCCESS) {
		return false ;
	}

	// Enumerate all of the decendents of this child.
	FILETIME time ;
	char szBuffer[256] ;
	DWORD dwSize = 256 ;
	while (RegEnumKeyEx(hKeyChild, 0, szBuffer, &dwSize, NULL,
	                    NULL, NULL, &time) == S_OK)
	{
		// Delete the decendents of this child.
		if(!RecursiveDeleteKey(hKeyChild, szBuffer)) {
			// Cleanup before exiting.
			RegCloseKey(hKeyChild) ;
			return false;
		}
		dwSize = 256 ;
	}

	// Close the child.
	RegCloseKey(hKeyChild) ;

	// Delete this child
	LONG result=RegDeleteKey(hKeyParent,KeyChild);
	return (result== ERROR_SUCCESS	|| result == ERROR_FILE_NOT_FOUND);
}


////////
//public
bool CWin32ImplRegistry::SetKeyAndValueNameAndValue(const char* Key,
										const char* Subkey,
										const char* ValueName,
										const char* Value)
{
	HKEY hSubKey;
	
	// Create and open key
	if(!GetHandleKey(Key,Subkey,&hSubKey)) {
		return false;
	}

	// Set the Value
	//FIXTODO : verifie les param de la fonction
	if(ValueName) {
		::RegSetValueEx(hSubKey, ValueName, 0, REG_SZ, 
		              (BYTE *)Value, 
		              strlen(Value)+1) ;
	}

	::RegCloseKey(hSubKey);
	return true;
}

void CWin32ImplRegistry::DeleteKey(const std::string& key_parent,const std::string& key_child)
{
	if(!RecursiveDeleteKey(m_hRootKey,GetAbsoluteKey(key_parent,key_child).c_str())) {
		hyp_THROW_COM(com_winregistry);
	}
	// Subkey may not exist...
}

void CWin32ImplRegistry::SetKey(const std::string& key_parent,const std::string& new_key)
{
	if(!SetKeyAndValueNameAndValue(key_parent.c_str(),new_key.c_str(),NULL,NULL)) {
		hyp_THROW_COM(com_winregistry);
	}
}

void CWin32ImplRegistry::SetValueNameAndValue(const std::string& key_parent,const std::string& key,
										 const std::string& value_name,const std::string& value)
{
	if(!SetKeyAndValueNameAndValue(key_parent.c_str(),key.c_str(),value_name.c_str(),value.c_str())) {
		hyp_THROW_COM(com_winregistry);
	}
}

/*
void CWin32ImplRegistry::SetKeyRoot(const std::string&)
{
	//pas implement
	assert(0);
}
*/

std::string CWin32ImplRegistry::GetValueFromValueName(const std::string& Key,
		const std::string& Subkey,
		const std::string& value_name)
{
	//on ouvre la Subkey
	HKEY hSubKey;
	if(!GetHandleKey(Key.c_str(),Subkey.c_str(),&hSubKey)) {
		hyp_THROW_COM(com_winregistry);
	}
	char szBuffer[256];
	DWORD dwSize=256;
	DWORD Type;

	LONG lResult=::RegQueryValueEx(
				hSubKey,           // handle to key to query
				value_name.c_str(),  // address of name of value to query
				NULL,  // reserved
				&Type,      // address of buffer for value type
				(LPBYTE)szBuffer,       // address of data buffer
				&dwSize     // address of data buffer size
				);
	
	if(lResult!=ERROR_SUCCESS) {
		hyp_THROW_COM(com_winregistry);
	}
	
	assert(Type==REG_SZ);

	::RegCloseKey(hSubKey);

	return std::string(szBuffer);
}


#endif	//#ifdef hyp_WIN32

//------------------------------------------------------------------------------
/*! \class CStdImplRegistry
\ingroup StdRegistry
Implements the IImplRegistry for the standard registry.
*/
//------------------------------------------------------------------------------
CStdImplRegistry::CStdImplRegistry()
{

}

void CStdImplRegistry::DeleteKey(const std::string& key_parent,const std::string& key_child)
{
	CStdRegistry reg(key_parent);
	reg.DeleteRegistryFile(key_child);
}

void CStdImplRegistry::SetKey(const std::string& key_parent,const std::string& new_key)
{
	CStdRegistry reg(key_parent);
	reg.CreateRegistryFile(new_key);
}

void CStdImplRegistry::SetValueNameAndValue(const std::string& key_parent,const std::string& key,
										 const std::string& value_name,const std::string& value)
{
	CStdRegistry reg(key_parent);
	reg.GetRegistryFile(key);
	reg.GetRegistryFile()->SetKey(value_name);
	reg.GetRegistryFile()->SetValue(value_name,value);
}

/*
void CStdImplRegistry::SetKeyRoot(const std::string&)
{
	//pas implement
	assert(0);
}
*/

std::string CStdImplRegistry::GetValueFromValueName(const std::string& key_parent,
		const std::string& key,
		const std::string& value_name)
{
	CStdRegistry reg(key_parent);
	reg.GetRegistryFile(key);
	return reg.GetRegistryFile()->GetValue(value_name);
}


hyp_NAMESPACE_END_HYPKERNEL
