/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Hyperion Pattern.
 Hyperion Pattern is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/

#ifndef _hyp_ker_reg_utility_h_
#define _hyp_ker_reg_utility_h_

#ifndef hyp_KERNEL_CORE_INSIDE
#error Inclusion not allowed (include hyp.kernel.core.h instead)
#endif


/*!
\ingroup Registry
Interface class. Used to implement differents systems to access to a registry system.
\todo Explain how the registry system works.
\todo SetKeyRoot
*/
class IImplRegistry
{
public:
	/*!
	Creates a new key.
	\param ParentKey Root key //Key which is the parent of the new key.
	\param Key Key that will be created.
	*/
	virtual void SetKey(const std::string& ParentKey,const std::string& Key) =0;

	/*!
	Deletes a key.
	\param ParentKey Root key
	\param Key Key that will be deleted
	*/
	virtual void DeleteKey(const std::string& ParentKey, const std::string& Key) =0;

	/*!
	Creates a value field and initalizes to its value.
	\param ParentKey Root key 
	\param Key Key where the field will be created
	\param ValueName Name of the created field.
	\param Value Value that initialize the created field.
	*/
	virtual void SetValueNameAndValue(const std::string& ParentKey,
		const std::string& Key,
		const std::string& ValueName,
		const std::string& Value) =0;

	/*!
	Get the value from a value field.
	\param ParentKey Root key.
	\param Key Key
	\param ValueName Field
	*/
	virtual std::string GetValueFromValueName(const std::string& ParentKey,
		const std::string& Key,
		const std::string& ValueName) =0;	
	
		
	//depend de l'implementation
	/*
	virtual void SetKeyRoot(const std::string&) =0;
	*/
	
};



class hyp_ker_DLL CRegistryUtility
{
	IImplRegistry* m_pImplRegistry;

public:
	/*!
	Enums the different registry systems which are possible to use.
	*/
	enum ERegistryType { 
		reg_win32,	//!< Uses the Win32 registry.
		reg_std		//!< Uses the standard registry.
	};

#ifdef hyp_USE_REGISTRY
	CRegistryUtility(ERegistryType =reg_win32);
#else
	CRegistryUtility(ERegistryType =reg_std);
#endif

	~CRegistryUtility();

	void RegisterServer(const std::string&, const CGuid&, const std::string&);
	void UnregisterServer(const CGuid&);
	std::string GetPathInprocessServer(const CGuid&);
	
	CGuid GetCLSIDFromProgID(const std::string&);
	std::string GetProgIDFromCLSID(const CGuid&);	
};


#endif
