/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Hyperion Pattern.
 Hyperion Pattern is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$

 Acknowledgements stx
 Creation 20/04/01 PR
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/


#ifndef _hyp_ker_time_h_
#define _hyp_ker_time_h_

#ifndef hyp_KERNEL_CORE_INSIDE
#error Inclusion not allowed (include hyp.kernel.core.h instead)
#endif

class hyp_ker_DLL CTime
{
	//! Number of seconds since 1/01/1970
	t_int64 m_Sec;

	//! Number of nano-secons to add to m_Sec
	t_ulong m_NanoSec;

public:
	CTime();	
	CTime(t_int64,t_ulong =0);

	/*! \name Extra figures
	Returns the number of X beyond the number of secondes.
	X represents :
	- milli-seconds
	- micro-seconds
	- nano-seconds
	*/
	//@{
	t_ulong GetExtraMilliSec() const { return m_NanoSec / 1000000UL; }
	t_ulong GetExtraMicroSec() const { return m_NanoSec / 1000UL; }
	t_ulong GetExtraNanoSec() const { return m_NanoSec; }
	//@}

  
	/*! \name 
	Return information about the current data before the call of the functions :
	- SetCourent
	- SetCurrentGMT	
	*/
	//@{
	t_int64 GetNanoSec() const   { return m_Sec * 1000000000UL + m_NanoSec; }
	t_int64 GetMicroSec() const   { return m_Sec * 1000000UL    + m_NanoSec / 1000UL; }
	t_int64 GetMilliSec() const   { return m_Sec * 1000UL       + m_NanoSec / 1000000UL; }
	t_int64 GetSec() const    { return m_Sec; }
	t_int64 GetMinute() const { return m_Sec /   60UL; }
	t_int64 GetHour() const   { return m_Sec /   3600UL; }
	t_int64 GetDay() const    { return m_Sec / ( 3600UL * 24UL ); }
	t_int64 GetWeek() const   { return m_Sec / ( 3600UL * 24UL * 7UL ); }
	//@}

	CTime& operator+=(const CTime& );
	CTime& operator-=(const CTime& );
	bool operator==(const CTime& ) const;
	bool operator!=(const CTime& ) const;
  
	bool IsInfinite() const;
	void SetInfinite();
	
	void SetCurrent();  
	void SetCurrentGMT();
	void SetCurrentLocalGMT();
	void Set(
		t_ulong,
		t_ulong,
		t_ulong,
		t_ulong = 0,
		t_ulong = 0,
		t_ulong = 0,
		t_ulong = 0 );
 

	t_ulong Year() const;
	t_ulong Month() const;
	t_ulong Day() const;
	t_ulong Hour() const;
	t_ulong Minute() const;
	t_ulong Second() const;
	t_ulong NanoSecond() const;
	
	friend hyp_ker_DLL std::ostream& operator<<(std::ostream&,const CTime&);	
};

#endif

