/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Hyperion Pattern.
 Hyperion Pattern is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$

 Acknowledgements stx
 Creation 19/03/01 PR
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/

#include"hyp.kernel.core.h"

#include<cstdio>
#include<cstdarg>

hyp_NAMESPACE_BEGIN_HYPKERNEL

/*! \defgroup Trace
@{
*/

/*!
Stream by default for the trace operations.
*/
std::ostream& gs_TraceStream=clog;

/*!
Get the stream used for the trace operations.
\return Trace stream
*/
hyp_ker_DLL std::ostream& GetTraceStream(void)
{
	return gs_TraceStream;
}

/*!
Set the stream used for the trace operations.
\param Stream Trace stream
*/
hyp_ker_DLL void SetTraceStream(std::ostream& Stream)
{
	gs_TraceStream=Stream;
}

/*!
Formats head data to a string. Used by the trace macro.
\param File File name where this function is insered. It is always filled by \c __FILE__ macro.
\param Line Line number where this function is insered. It is always filled by \c __LINE__ macro.
\return Formated string.
*/
hyp_ker_DLL std::string Begin( const char *File, int Line )
{
	if(File!=0) {
		std::stringstream stream;
		stream<<"\n->\n["<<File<<"-"<<Line<<"]\n";
		return stream.str();
	} else return std::string();
}

/*!
Formats the end date to a string. Used by the trace macro.
\return Formated string.
*/
hyp_ker_DLL std::string End(void)
{
  	return std::string("<-\n");
}

/*!
Formats data to a string.
\param Msg Message made up of arguments from the printf function.
\return Formated string
\todo overflow capacity
*/
hyp_ker_DLL std::string Print( const char* Msg, ... )
{
	const int size_buffer=256;
	char buffer[size_buffer];
	va_list arglist;
	va_start(arglist, Msg);
	
	if(::_vsnprintf(buffer,size_buffer,Msg,arglist)==-1) {
	}

	va_end(arglist);
	std::string str(buffer);
	str+='\n';
	return str;
}


/*@}*/


hyp_NAMESPACE_END_HYPKERNEL