/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Hyperion Pattern.
 Hyperion Pattern is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$

 Acknowledgements stx
 Creation 19/03/01 PR
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/



#ifndef _hyp_ker_trace_h_
#define _hyp_ker_trace_h_

#ifndef hyp_KERNEL_CORE_INSIDE
#error Inclusion not allowed (include hyp.kernel.core.h instead)
#endif

hyp_ker_DLL std::ostream& GetTraceStream(void);
hyp_ker_DLL void SetTraceStream(std::ostream&);
hyp_ker_DLL std::string Begin(const char*, int);
hyp_ker_DLL std::string Print(const char*,... );
hyp_ker_DLL std::string End(void);

/*! \defgroup Trace
@{
*/

/*!
Defines the stream used to trace operations. This stream is called <i>trace file</i>.
By default, the stream is the <i>hyplog file</i>.
*/
#define hyp_TRACE_SET_FILE( Stream )            \
	hyp::ker::SetStreamTrace( Stream )

/*!
Standard trace macro.
*/
#define hyp_TRACE( args )                       \
{	                                            \
	hyp_TRACE_BEGIN;                            \
	hyp_TRACE_DO( args );                       \
	hyp_TRACE_END;                              \
}

/*!
Writes in the <i>trace file</i>:
- the file name;
- the line number where the macro has been insered.
*/
#define hyp_TRACE_BEGIN                         \
	hyp::ker::GetTraceStream()<<hyp::ker::Begin( __FILE__, __LINE__ )

/*!
Writes in the <i>trace file</i> a message which is made up of arguments from the printf function.
\code
hyp_TRACE_DO( ( "toto, %s: %d,%d", "hello", 3, 4 ) );
\endcode
*/
#define hyp_TRACE_DO( Args )                    \
	hyp::ker::GetTraceStream()<<hyp::ker::Print Args

/*!
Writes in the <i>trace file</i> a sequence of characters which terminate the trace operations.
*/
#define hyp_TRACE_END                           \
	hyp::ker::GetTraceStream()<<hyp::ker::End()

/*@}*/

/*! \defgroup StringTrace
\ingroup Trace
@{
*/

#define hyp_STRING_TRACE( String , Args )       \
{	                                            \
	hyp_STRING_BEGIN( String );                 \
	hyp_STRING_DO( String , Args );             \
	hyp_STRING_END( String );                   \
}

#define hyp_STRING_BEGIN( String )              \
	String+=hyp::ker::Begin( __FILE__, __LINE__ )

#define hyp_STRING_DO( String, Args )           \
	String+=hyp::ker::Print Args

#define hyp_STRING_END( String )               \
	String+=hyp::ker::End()

/*@}*/

#endif