/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Hyperion Pattern.
 Hyperion Pattern is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/


#ifndef _hypstd_factory_h_
#define _hypstd_factory_h_


#ifndef hyp_KERNEL_STD_INSIDE
#error Inclusion not allowed (include hyp.kernel.std.h instead)
#endif


class CFactory :
public IClassFactory
{
public:
	CFactory(const CFactoryData* );
	~CFactory();

	//! IUnknown Implementation
	//@{
	virtual com_result QueryInterface(const hypIID&, void**);
	virtual t_ulong AddRef();
	virtual t_ulong Release();
	//@}

	//! IClassFactory implementation
	//@{
	virtual com_result CreateInstance(IUnknown*, const hypIID& iid, void**);
	virtual com_result LockServer(bool); 
	//}

	
	static com_result GetClassObject(const hypCLSID&,const hypIID&, void**);
	static bool IsLocked();
	static com_result RegisterAll();
	static com_result UnregisterAll();
	static com_result CanUnloadNow();
	
	static void InitStaticMembers(const CFactoryData*,
								  int,
								  const std::string&);

	
private:
	//typedef
	typedef std::vector<CFactoryData> ContFactoryData;
	typedef ContFactoryData::iterator ContFactoryDataItor;
	typedef ContFactoryData::iterator ContFactoryDataConstItor;
	
	//compteur de rference
	long m_cRef;

	//pointeur vers un lment du conteneur ms_FactoryData
	//Ce pointeur permet d'associer une fabrique  un composant
	const CFactoryData* m_pData;

	//Conteneur vers toutes les informations necessaires pour la gestion des
	//composants se trouvant hebergs sur ce serveur
	static ContFactoryData ms_FactoryData;

	//Compteur de verrou
	static long ms_cServerLocks;

	//Ce nom est l'identifiant du serveur qui pour toute les plateforme est
	//le nom complet du serveur.On s'affranchit des handles divers suivant les OS...
	static std::string ms_NameModule;
};


/*!
\ingroup StdComponent
Include this macro in a implementation files to define automatically the exported functions.
Don't miss to define a .def file to export these functions.
*/
#define hyp_DECLARE_EXPORTED_FUNCTIONS                                               \
	hyp_EXPORT_DLL hyp_ker::com_result hyp_CALL_CONV hypDllCanUnloadNow(void) {      \
		return hyp_ker::CFactory::CanUnloadNow();                                    \
	}                                                                                \
	hyp_EXPORT_DLL hyp_ker::com_result hyp_CALL_CONV hypDllGetClassObject(const hyp_ker::hypCLSID& clsid,const hyp_ker::hypIID& iid,void** ppv) { \
		hypstd_TRACE( ("Appel de DllGetClassObject") );                              \
		return hyp_ker::CFactory::GetClassObject(clsid, iid, ppv);                   \
	}                                                                                \
	hyp_EXPORT_DLL hyp_ker::com_result hyp_CALL_CONV hypDllRegisterServer(void) {    \
		return hyp_ker::CFactory::RegisterAll();                                     \
	}                                                                                \
	hyp_EXPORT_DLL hyp_ker::com_result hyp_CALL_CONV hypDllUnregisterServer(void) {  \
		return hyp_ker::CFactory::UnregisterAll();                                   \
	}


#endif