/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Hyperion Pattern.
 Hyperion Pattern is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/


#ifndef _hypstd_factorydata_h_
#define _hypstd_factorydata_h_


#ifndef hyp_KERNEL_STD_INSIDE
#error Inclusion not allowed (include hyp.kernel.std.h instead)
#endif

/*!
\ingroup StdComponent
Contents some important date about a component. Must be initialized when the component server
is loaded. This initialization is done typically with the InitLibrary function.
*/
class CFactoryData
{
public:
	/*!
	Defines a function pointer to a function CreateInstance that must be defined by each component.
	*/
	typedef com_result (*PTR_CREATEINSTANCE)(IUnknown*,CUnknown**);
	

	CFactoryData(const hypCLSID* pCLSID,
				 PTR_CREATEINSTANCE CreateInstance,
				 const std::string& ComponentName) :
	m_pCLSID(pCLSID),
	CreateInstance(CreateInstance),
	m_ComponentName(ComponentName) {
	}

	CFactoryData(const CFactoryData& aFactoryData)
	{
		m_pCLSID=aFactoryData.m_pCLSID;
		CreateInstance=aFactoryData.CreateInstance;
		m_ComponentName=aFactoryData.m_ComponentName;
	}

	~CFactoryData() { }
	
	/*!
	Help function used to find the CLSID of the component.
	*/
	bool IsClassID(const hypCLSID& clsid) const
	{ return clsid==(*m_pCLSID); }

	//! Pointer to the component CLSID.
	const hypCLSID* m_pCLSID;

	//! Pointer to the function that create the component.
	PTR_CREATEINSTANCE CreateInstance;
	
	//!  Friendly name of the component
	std::string m_ComponentName;
};


#endif