/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Hyperion Pattern.
 Hyperion Pattern is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$

 Creation 05/07/2000 PR
 Update 28/07/00 CreateInstance added PR
 Acknowledgements Dale Rogerson
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/


#ifndef _hypstd_smart_pointeur_h_
#define _hypstd_smart_pointeur_h_

#ifndef hyp_KERNEL_INSIDE
#error Inclusion not allowed (include hyp.kernel.h instead)
#endif

/*!
\ingroup SmartPointer
Don't use with IUnknown i.e. \c IPtr<IUnknown,&IID_hypIUnknown>, use \c IPtrUnknown.
\todo Specialisation de IPtr pour l'adpater  IPtrUnknown
\todo T* operator=(psco::IUnknown* pI) lancement exception?
*/
template<class T, const hypIID* piid>
class IPtr
{
	T* m_pI;	
public:
	IPtr() :
	m_pI(0) {
	
	}


	IPtr(T* lp) :
	m_pI(lp) {
		if(m_pI!=0) {
			m_pI->AddRef();
		}
	}

	IPtr(IUnknown* pI) :
	m_pI(0)	{
		m_pI=0;
		if(pI!=0) {
			pI->QueryInterface(*piid,(void**)&m_pI);
		}
	}

	~IPtr()	{
		Release();
	}

	//! Reset
	void Release() {
		if(m_pI!=0) {
			T* pOld=m_pI;
			m_pI=0;
			pOld->Release();
		}
	}

	inline operator T*() {
		return m_pI;
	}

	inline operator T*() const {
		return m_pI;
	}

	inline const T* get() const {
		return m_pI;
	}

	inline T& operator*() 	{
		hyp_ASSERT(m_pI);
		return *m_pI;
	}
	
	inline T** operator&() {
		hyp_ASSERT(m_pI==0);
		return &m_pI;
	}
	
	inline T* operator->() {
		hyp_ASSERT(m_pI);
		return m_pI;
	}

	//! Assigns to the same interface.
	T* operator=(T* pI)	{
		if(m_pI!=pI) {
			IUnknown* pOld=m_pI;
			m_pI=pI;

			if(m_pI!=0) {
				m_pI->AddRef();
			}
			if(pOld!=0) {
				pOld->Release();
			}
		}
		return m_pI;
	}

	//! Assigns to an another interface.
	T* operator=(IUnknown* pI) {
		IUnknown* pOld=m_pI;
		m_pI=0;

		if(pI!=0) {
			com_result hr=pI->QueryInterface(*piid,(void**)&m_pI);
			if(hr!=com_ok) {
				m_pI=0;
				//FIXTODO : lancement d'exeption????
				hyp_ASSERT(0);
			}
		}

		if(pOld!=0) {
			pOld->Release();
		}
		return m_pI;
	}

	inline bool operator!() {
		return (m_pI==0)? true:false;
	}

	inline const hypIID& iid() {
		return *piid;
	}

	//! Methods to create type-safe components.
	void CreateInstance(const hypCLSID& clsid,IUnknown* pI,long clsctx = INPROC_SERVER) {
		Release();
		if(Failed(ker::CreateInstance(clsid,pI,clsctx,*piid,(void**)&m_pI))) {
			hyp_THROW_COM(com_false);
		}
	}
};



/*!
\ingroup SmartPointer
IPtrUnknown is a smart interface for IUnknown.
*/
class IPtrUnknown
{
	IUnknown* m_pI;
public:
	IPtrUnknown() :
	m_pI(0)	{
		
	}

	IPtrUnknown(IUnknown* lp) {
		m_pI=lp;
		if(m_pI!=0) {
			m_pI->AddRef();
		}
	}

	~IPtrUnknown() { 
		Release();
	}

	void Release() {
		if (m_pI) {
			IUnknown* pOld = m_pI ;
			m_pI = 0;
			pOld->Release() ;
		}
	}

	operator IUnknown*(){
		return static_cast<IUnknown*>(m_pI);
	}

	IUnknown& operator*() {
		hyp_ASSERT(m_pI);
		return *m_pI;
	}

	IUnknown** operator&() {
		hyp_ASSERT(m_pI==0);
		return &m_pI;
	}
	
	IUnknown* operator->() {
		hyp_ASSERT(m_pI);
		return m_pI;
	}

	IUnknown* operator=(IUnknown* pI) {		
		if(m_pI!=pI) {
			IUnknown* pOld=m_pI;
			m_pI=pI;
			if(m_pI!=0) {
				m_pI->AddRef();
			}
			if(pOld!=0) {
				pOld->Release() ;
			}
		}
		return m_pI;
	}

	bool operator!() {
		return (m_pI==0) ? true : false;
	}

	inline const hypIID& iid() {
		return IID_hypIUnknown;
	}

	com_result CreateInstance(const hypCLSID& clsid,IUnknown* pI,long clsctx = INPROC_SERVER) {
		Release();
		return ker::CreateInstance(clsid,pI,clsctx,IID_hypIUnknown,(void**)&m_pI);
	}
};


/*!
Conversion operator like static_cast or dynamic_cast
\todo definir cet operateur  porter global???
*/
template <class I, const hypGUID* pGUID>
I* interface_cast(IUnknown* pIUnknown)
{
	I* pI=0;
	com_result hr=pIUnknown->QueryInterface(*pGUID,(void**)&pI);
	hyp_ASSERT(hr==com_ok);
	return pI;
}


#endif	