/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Hyperion Pattern.
 Hyperion Pattern is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/


#include"hyp.kernel.h"
#include"hyp.kernel.std.h"

hyp_NAMESPACE_BEGIN_HYPKERNEL


//---------------------------------------------------------
/*! \class CUnknown
\ingroup StdComponent
IUnknown Implementation by default. Derive your component from
this class.
\todo example for more information.
*/	
//---------------------------------------------------------

/*!
Actifs components instanciated by the server.
Is used to check if the server can be unloaded.
*/
long CUnknown::ms_cActiveComponents = 0;


CUnknown::CUnknown(IUnknown* pUnknownOuter)
: m_cRef(1)
{
	// Set m_pUnknownOuter pointer.
	if (pUnknownOuter == 0)
	{
		hypstd_TRACE( ("CU :\tNot aggregating; delegate to nondelegating IUnknown.") );
		m_pUnknownOuter = reinterpret_cast<IUnknown*>
		                     (static_cast<INondelegatingUnknown*>
		                     (this)) ;  // notice cast !!!!!!!!!!!!!!!!!!!!!!!! ultra important
	}
	else
	{
		hypstd_TRACE( ("CU :\tAggregating; delegate to outer IUnknown.") );
		m_pUnknownOuter = pUnknownOuter ;
	}

	// Increment count of active components.
	++ms_cActiveComponents;
}


CUnknown::~CUnknown()
{
	--ms_cActiveComponents;
}

/*!
Notifies the final release of the derived class. Called by Release funcion.
Can be overwrited.
*/
void CUnknown::FinalRelease()
{
	hypstd_TRACE( ("CU :\tIncrement reference count for final release.") ); 
	m_cRef = 1 ;
}

/*!
Must be overwrite if the component supports another interfaces than IUnknown.
*/
com_result CUnknown::NondelegatingQueryInterface(const hypIID& iid,void** ppv)										  
{
	if (iid == IID_hypIUnknown)	// CUnknown supports only IUnknown
	{
		return FinishQI(reinterpret_cast<IUnknown*>
		                   (static_cast<INondelegatingUnknown*>(this)), ppv) ;
	}	
	else
	{
		*ppv =0 ;
		return com_no_interface;
	}
}


t_ulong CUnknown::NondelegatingAddRef()
{
	return ++m_cRef;
}

t_ulong CUnknown::NondelegatingRelease()
{
	--m_cRef;
	if (m_cRef == 0)
	{
		FinalRelease() ;
		delete this ;
		return 0 ;
	}
	return m_cRef ;
}

/*!
Assignes two pointers to a same interface.
So the reference counter of the interface is incremented.
*/
com_result CUnknown::FinishQI(IUnknown* pI, void** ppv) 
{
	*ppv = pI ;
	pI->AddRef() ;
	return com_ok;
}


hyp_NAMESPACE_END_HYPKERNEL
