/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Hyperion Pattern.
 Hyperion Pattern is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/


#ifndef _hypstd_unknwn_h_
#define _hypstd_unknwn_h_


class CUnknown :
public INondelegatingUnknown
{
public:
	CUnknown(IUnknown*);
	virtual ~CUnknown();

	//! INondelegatingUnknown Implementation
	//@{
	virtual com_result NondelegatingQueryInterface(const hypIID&, void**);
	virtual t_ulong NondelegatingAddRef();
	virtual t_ulong NondelegatingRelease();
	//@}

	/*! Initialisation function used for the aggregation cases.
	Can be overwrite.
	*/
	virtual com_result Init() {
		return com_ok;
	}
	virtual void FinalRelease();
	
	/*!
	Returns the number of actives components.
	*/
	inline static long ActiveComponents() { 
		return ms_cActiveComponents;
	}
	
	com_result FinishQI(IUnknown*, void**);
	
protected:
	/*!
	Gets the aggregated component.
	*/
	inline IUnknown* GetOuterUnknown() const {
		return m_pUnknownOuter;
	}
	
	long m_cRef;	//!< Reference counter of the component.
	IUnknown* m_pUnknownOuter;	//!< Pointer to a component which aggragates this object.
	static long ms_cActiveComponents;
};


/*!
\ingroup StdComponent
Utility macro used to defined the functions of the IUnknown interface.
*/
#define hyp_DECLARE_IUNKNOWN                                                               \
	virtual hyp::ker::com_result QueryInterface(const hyp::ker::hypIID & iid, void** ppv) {\
		return GetOuterUnknown()->QueryInterface(iid,ppv);                                 \
	}                                                                                      \
	virtual hyp::t_ulong AddRef() {                                                        \
		return GetOuterUnknown()->AddRef();                                                \
	}                                                                                      \
	virtual hyp::t_ulong Release() {                                                       \
		return GetOuterUnknown()->Release();                                               \
	}


#endif