/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Hyperion Pattern.
 Hyperion Pattern is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$

 Creation 14/03/01 PR
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/

#ifndef _hyp_type_h_
#define _hyp_type_h_

#include"hyp.def.h"
#include"hyp.ker.def.h"

hyp_NAMESPACE_BEGIN_HYP

/*! \file hyp.type.h
Include this file to:
- have only the general macros;
- use the usual predefined types of the <i>Hyperion</i> namespace.
*/

/*! \defgroup GeneralType General Types
\ingroup GeneralDef
*/
//@{

typedef int t_label;
typedef unsigned int t_size;

typedef unsigned short int t_ushort;
typedef unsigned char t_uchar;
typedef unsigned long int t_ulong;
typedef unsigned int t_uint;
//@}

/*! \defgroup RealType Reals
\ingroup GeneralType
If the macro \c hyp_USE_DOUBLE is defined, Hyperion will uses exclusively
32bits reals.
*/ 
//@{
#define hyp_USE_DOUBLE 1

#ifdef hyp_USE_DOUBLE
	typedef double t_real;
#else 
	typedef float t_real;
#endif
//@}


/*!
\ingroup GeneralType
Defines flags.
\todo verif de la limite de pos + operateur de convertion
*/
class CFlag
{	
public:
	typedef t_uint t_flag;
	
	explicit CFlag(const t_size& Pos) { m_Flag = (0x00000001<<Pos) ; }
	CFlag() : m_Flag(0) {};

	inline operator t_flag() const {
		return GetFlag();
	}
	inline t_flag GetFlag(void) const {
		return m_Flag;
	}

	inline void SetFlag(const t_flag& Flag) {
		m_Flag|=Flag;
	}	
	inline bool HaveFlag(const t_flag& Flag) const {
		return (Flag & m_Flag) ==0 ? false : true;
	}

private:
	t_flag m_Flag;
};


/*! \defgroup ArrayType Arrays
\ingroup GeneralType
*/
//@{
/*!
Defines arrays. In fact, this class is a trick to switch easily between two standard containers :
- std::vector
- std::array
To switch, just define the macro \c USE_VALARRY and TArray will use \c std::array .
*/
template<class T>
class TArray :
#ifdef hyp_USE_VALARRAY
public std::valarray<T>
#else
public std::vector<T>
#endif
{
public:
	/*!
	Constructs an array.
	\param Capacity Capacity of the created array.
	\param Data Value which is used to initialize the entire array.
	*/
	explicit TArray(t_size Capacity,const T& Data=T()) {
		resize(Capacity,Data);
	}

	/*!
	Resets the entire array.
	\param Data Value which is used to initialize the entire array.
	*/
	void reset(const T& Data) {
		for(int i=0;size();i++)	(*this)[i]=Data;
	}

	/*!
	Shows the array in a defined stream.
	\param Stream Stream which is used to show the array.
	*/
	void show(std::ostream& Stream) {
		stream<<"TArray : "<<this<<" - Size : "<<size()<<"\n";
		for(int i=0;i<size();i++) {
			Stream<<"N : "<<i<<" - "<<at(i)<<"\n";
		}
	}
};

typedef TArray<t_real> t_real_array;
typedef TArray<bool> t_mask_array;
typedef TArray<t_label> t_label_array;

typedef std::vector<t_label> t_label_enum;
//@}


/*! \defgroup Int64Type 64 bits Integers
\ingroup GeneralType
*/
//@{
typedef signed __int64 t_int64;
hyp_ker_DLL t_int64 GetInt64(t_uint High,t_uint Low);
hyp_ker_DLL t_uint GetHighInt64(t_int64);
hyp_ker_DLL t_uint GetLowInt64(t_int64);
hyp_ker_DLL std::ostream& operator<<(std::ostream&,const t_int64&);
//@}



hyp_NAMESPACE_END_HYP

#endif